import 'package:example/main.dart';
import 'package:fl_camera/fl_camera.dart';
import 'package:flutter/material.dart';
import 'package:flutter_waya/flutter_waya.dart';

class CameraPage extends StatefulWidget {
  const CameraPage({Key? key}) : super(key: key);

  @override
  _CameraPageState createState() => _CameraPageState();
}

class _CameraPageState extends State<CameraPage> {
  bool flashState = false;
  double ratio = 1;
  double? maxRatio;
  StateSetter? flashZoomState;
  FlCameraController? controller;

  @override
  Widget build(BuildContext context) {
    return ExtendedScaffold(
        body: Stack(children: <Widget>[
      FlCamera(
          onCreateView: (FlCameraController _controller) {
            controller = _controller;
          },
          fit: BoxFit.fitHeight,
          onFlashChanged: (FlashState state) {
            showToast('flash state:$state');
          },
          onZoomChanged: (CameraZoomState zommState) {
            showToast('zoom ratio:${zommState.zoomRatio}');
            if (maxRatio == null && flashZoomState != null) {
              maxRatio = zommState.maxZoomRatio;
              flashZoomState!(() {});
            }
          },
          overlay: Align(
              alignment: Alignment.bottomCenter,
              child: StatefulBuilder(builder: (_, StateSetter state) {
                flashZoomState = state;
                return Column(
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      Slider(
                          value: ratio,
                          min: 1,
                          max: maxRatio ?? 20,
                          onChanged: (double value) async {
                            ratio = value;
                            flashZoomState!(() {});
                            FlCameraController().setZoomRatio(value);
                          }),
                      IconBox(
                          size: 30,
                          color: flashState
                              ? Colors.white
                              : Colors.white.withOpacity(0.6),
                          padding: const EdgeInsets.fromLTRB(10, 10, 10, 40),
                          icon: flashState ? Icons.flash_on : Icons.flash_off,
                          onTap: () async {
                            final bool state = await FlCameraController()
                                .setFlashMode(flashState
                                    ? FlashState.off
                                    : FlashState.on);
                            if (state) {
                              flashState = !flashState;
                              flashZoomState!(() {});
                            }
                          })
                    ]);
              }))),
      Positioned(
          right: 12,
          left: 12,
          top: getStatusBarHeight + 12,
          child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                const BackButton(color: Colors.white, onPressed: pop),
                ValueBuilder<bool>(
                    initialValue: controller?.cameraOptions != null,
                    builder: (_, bool? value, ValueCallback<bool> updater) {
                      value ??= false;
                      return ElevatedText(
                        text: value ? 'start' : 'stop',
                        onPressed: () async {
                          if (controller == null) return;
                          bool state = false;
                          if (value!) {
                            if (controller!.previousCameraId != null) {
                              final options = await controller!
                                  .startPreview(controller!.previousCameraId!);
                              state = options != null;
                            }
                          } else {
                            state = await controller!.stopPreview();
                          }
                          if (state) updater(!value);
                        },
                      );
                    }),
              ])),
    ]));
  }
}

class ScannerBoxPage extends StatelessWidget {
  const ScannerBoxPage({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return const ExtendedScaffold(
        body: FlCamera(
            overlay: ScannerBox(
                borderColor: Colors.blue,
                scannerColor: Colors.blue,
                scannerSize: Size(300, 300),
                scannerStrokeWidth: 2)));
  }
}

class ScannerLinePage extends StatelessWidget {
  const ScannerLinePage({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return const ExtendedScaffold(
        body: FlCamera(overlay: ScannerLine(color: Colors.blue)));
  }
}
